﻿/*Agnitio security code review tool.
Copyright (C) 2011 David Rook

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Text.RegularExpressions;
using System.Data.SQLite;

namespace Agnitio
{
    public partial class Rules : Form
    {
        public Rules()
        {
            InitializeComponent();
        }

        private void Rules_Load(object sender, EventArgs e)
        {
            kryptonDataGridView1.Columns[0].HeaderCell.Style.Alignment = DataGridViewContentAlignment.MiddleCenter;
            kryptonDataGridView1.Columns[0].DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleCenter;
            kryptonDataGridView1.Columns[1].HeaderCell.Style.Alignment = DataGridViewContentAlignment.MiddleCenter;
            kryptonDataGridView1.Columns[2].HeaderCell.Style.Alignment = DataGridViewContentAlignment.MiddleCenter;
            
            populateDropdown();

            loadQuestions();
        }

        string dPath = Form1.dPath.ToString();
        string rPath = Form1.rPath.ToString();

        private void languageCombo_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (!Regex.IsMatch(e.KeyChar.ToString(), @"^ . $"))
                e.Handled = true;
        }

        private void keywordBox_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (!Regex.IsMatch(e.KeyChar.ToString(), @"^[a-zA-Z0-9àáâãäåăćĉċčçèéêëēĕėěğĝġĥìíîïĩĭĵĺñńňòóôõöōŏőŕřśŝšşùúûüũūŭűŵŷýÿźżžÀÁÂÃÄÅĂßĆĈĊČÇÈÉÊËĒĔĖĚĞĜĠĤÌÍÎÏĨĬĹĴÑŃŇÒÓÔÕÖŌŎŐŔŘŚŜŠŞÙÚÛÜŨŪŬŰŴŶÝŸŹŻŽ\-_.,*?¡¿%{}()\[\]\\\/$:;+&@|~#""!&=<>^\b\s]*$"))
                e.Handled = true;
        }

        private void languageBox_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (!Regex.IsMatch(e.KeyChar.ToString(), @"^[a-zA-Z0-9àáâãäåăćĉċčçèéêëēĕėěğĝġĥìíîïĩĭĵĺñńňòóôõöōŏőŕřśŝšşùúûüũūŭűŵŷýÿźżžÀÁÂÃÄÅĂßĆĈĊČÇÈÉÊËĒĔĖĚĞĜĠĤÌÍÎÏĨĬĹĴÑŃŇÒÓÔÕÖŌŎŐŔŘŚŜŠŞÙÚÛÜŨŪŬŰŴŶÝŸŹŻŽ\-_.,*?¡¿%{}()\[\]\\\/$:;+&@|~#""!&=<>^\b\s]*$"))
                e.Handled = true;
        }

        private void loadQuestions()
        {
            SQLiteConnection ObjConnection = new SQLiteConnection("Data Source=" + dPath + ";Version=3;New=False;Compress=True;FailIfMissing=True");
            string commandtext = ("SELECT question, Principle FROM questions");
            
            try
            {
                ObjConnection.Open();
            }

            catch
            {
                DevExpress.XtraEditors.XtraMessageBox.Show("Unable to load questions from the database");
                return;
            }

            try
            {
                SQLiteDataAdapter dataAdapter = new SQLiteDataAdapter(commandtext, ObjConnection);
                DataSet ds = new DataSet();
                dataAdapter.Fill(ds);

                kryptonDataGridView1.Columns["Principle"].Frozen = false;
                kryptonDataGridView1.Columns["question"].Frozen = false;

                kryptonDataGridView1.DataSource = ds.Tables[0];
                kryptonDataGridView1.Columns["Principle"].DisplayIndex = 0;
                kryptonDataGridView1.Columns["question"].DisplayIndex = 1;
                
            }

            catch
            {
                DevExpress.XtraEditors.XtraMessageBox.Show("Unable to populate the checklist grid");
                return;
            }

            ObjConnection.Close();
        }

        private void populateDropdown()
        {
            SQLiteConnection ObjConnection = new SQLiteConnection("Data Source=" + dPath + ";Version=3;New=False;Compress=True");

            try
            {
                ObjConnection.Open();
                SQLiteDataReader rdr = null;

                languageCombo.Properties.Items.Clear();

                SQLiteCommand cmd = new SQLiteCommand("SELECT DISTINCT language FROM reviewrules", ObjConnection);
                rdr = cmd.ExecuteReader();

                {
                    while (rdr.Read())
                    {
                        for (int i = 0; i < rdr.FieldCount; i++)
                        {
                            languageCombo.Properties.Items.Add(rdr.GetValue(i).ToString());
                        }
                    }
                }
            }

            catch
            {
                DevExpress.XtraEditors.XtraMessageBox.Show("Unable to retrieve languages from the database");
                return;
            }

            ObjConnection.Close();
        }

        private void languageCombo_TextChanged(object sender, EventArgs e)
        {
            try
            {
                if (!Regex.IsMatch(languageBox.Text, @"^[a-zA-Z0-9àáâãäåăćĉċčçèéêëēĕėěğĝġĥìíîïĩĭĵĺñńňòóôõöōŏőŕřśŝšşùúûüũūŭűŵŷýÿźżžÀÁÂÃÄÅĂßĆĈĊČÇÈÉÊËĒĔĖĚĞĜĠĤÌÍÎÏĨĬĹĴÑŃŇÒÓÔÕÖŌŎŐŔŘŚŜŠŞÙÚÛÜŨŪŬŰŴŶÝŸŹŻŽ\-_.,*?¡¿%{}()\[\]\\\/$:;+&@|~#""!&=<>^\b\s]*$"))
                    throw new Exception("Invalid characters in the language text box");
            }
            catch (Exception ex)
            {
                DevExpress.XtraEditors.XtraMessageBox.Show(ex.Message);
                return;
            }

            string lang = languageCombo.Text.ToString();
            
            SQLiteConnection ObjConnection = new SQLiteConnection("Data Source=" + dPath + ";Version=3;New=False;Compress=True");

            try
            {
                ObjConnection.Open();
            }

            catch
            {
                DevExpress.XtraEditors.XtraMessageBox.Show("Could not connect to the database");
                return;
            }

            try
            {
                int keywordcount = 0;

                try
                {
                    SQLiteCommand countcommand = new SQLiteCommand("SELECT COUNT(keyword) FROM reviewrules WHERE language = '" + lang.ToString() + "' ", ObjConnection);
                    string rowCount = countcommand.ExecuteScalar().ToString();
                    keywordcount = Convert.ToInt32(rowCount);
                }

                catch
                {
                    DevExpress.XtraEditors.XtraMessageBox.Show("Could not connect to the database");
                    return;
                }

                listView1.Items.Clear();

                DataSet ds = new DataSet();
                SQLiteDataAdapter adapter = new SQLiteDataAdapter("SELECT keyword FROM reviewrules WHERE language = '" + lang.ToString() + "' ", ObjConnection);
                adapter.Fill(ds);

                foreach (DataRow row in ds.Tables[0].Rows)
                {
                    listView1.Items.Add(row["keyword"].ToString());
                }
            }

            catch
            {
                DevExpress.XtraEditors.XtraMessageBox.Show("Could not connect to the database");
                return;
            }

            ObjConnection.Close();
        }

        string selectedKeyword = "";
        int num = 0;

        private void listView1_Click(object sender, EventArgs e)
        {
            try
            {
                ListView.SelectedListViewItemCollection keyword = this.listView1.SelectedItems;

                foreach (ListViewItem item in keyword)
                {
                    selectedKeyword = (item.Text);
                }
            }

            catch
            {
                DevExpress.XtraEditors.XtraMessageBox.Show("Unable to select item from the keyword list");
                return;
            }

            if (selectedKeyword != "")
            {
                try
                {
                    if (!Regex.IsMatch(keywordBox.Text, @"^[a-zA-Z0-9àáâãäåăćĉċčçèéêëēĕėěğĝġĥìíîïĩĭĵĺñńňòóôõöōŏőŕřśŝšşùúûüũūŭűŵŷýÿźżžÀÁÂÃÄÅĂßĆĈĊČÇÈÉÊËĒĔĖĚĞĜĠĤÌÍÎÏĨĬĹĴÑŃŇÒÓÔÕÖŌŎŐŔŘŚŜŠŞÙÚÛÜŨŪŬŰŴŶÝŸŹŻŽ\-_.,*?¡¿%{}()\[\]\\\/$:;+&@|~#""!&=<>^\b\s]*$"))
                        throw new Exception("Invalid characters in the keyword text box");
                    if (!Regex.IsMatch(languageBox.Text, @"^[a-zA-Z0-9àáâãäåăćĉċčçèéêëēĕėěğĝġĥìíîïĩĭĵĺñńňòóôõöōŏőŕřśŝšşùúûüũūŭűŵŷýÿźżžÀÁÂÃÄÅĂßĆĈĊČÇÈÉÊËĒĔĖĚĞĜĠĤÌÍÎÏĨĬĹĴÑŃŇÒÓÔÕÖŌŎŐŔŘŚŜŠŞÙÚÛÜŨŪŬŰŴŶÝŸŹŻŽ\-_.,*?¡¿%{}()\[\]\\\/$:;+&@|~#""!&=<>^\b\s]*$"))
                        throw new Exception("Invalid characters in the language text box");
                }
                catch (Exception ex)
                {
                    DevExpress.XtraEditors.XtraMessageBox.Show(ex.Message);
                    return;
                }

                try
                {
                    keywordBox.Text = selectedKeyword.ToString();
                    languageBox.Text = languageCombo.Text.ToString();
                    loadQuestions();

                    SQLiteConnection ObjConnection = new SQLiteConnection("Data Source=" + dPath + ";Version=3;New=False;Compress=True");

                    ObjConnection.Open();

                    SQLiteCommand cmd = new SQLiteCommand("SELECT info FROM reviewrules WHERE keyword = '" + keywordBox.Text.ToString() + "' AND language = '" + languageBox.Text.ToString() + "' ", ObjConnection);
                    ruleInfoBox.Text = cmd.ExecuteScalar().ToString();

                    SQLiteCommand cmd2 = new SQLiteCommand("SELECT linked1 FROM reviewrules WHERE keyword = '" + keywordBox.Text.ToString() + "' AND language = '" + languageBox.Text.ToString() + "' ", ObjConnection);
                    string linked1 = cmd2.ExecuteScalar().ToString();

                    SQLiteCommand cmd3 = new SQLiteCommand("SELECT linked2 FROM reviewrules WHERE keyword = '" + keywordBox.Text.ToString() + "' AND language = '" + languageBox.Text.ToString() + "' ", ObjConnection);
                    string linked2 = cmd3.ExecuteScalar().ToString();

                    SQLiteCommand cmd4 = new SQLiteCommand("SELECT linked3 FROM reviewrules WHERE keyword = '" + keywordBox.Text.ToString() + "' AND language = '" + languageBox.Text.ToString() + "' ", ObjConnection);
                    string linked3 = cmd4.ExecuteScalar().ToString();

                    SQLiteCommand cmd5 = new SQLiteCommand("SELECT linked4 FROM reviewrules WHERE keyword = '" + keywordBox.Text.ToString() + "' AND language = '" + languageBox.Text.ToString() + "' ", ObjConnection);
                    string linked4 = cmd5.ExecuteScalar().ToString();

                    SQLiteCommand cmd6 = new SQLiteCommand("SELECT linked5 FROM reviewrules WHERE keyword = '" + keywordBox.Text.ToString() + "' AND language = '" + languageBox.Text.ToString() + "' ", ObjConnection);
                    string linked5 = cmd6.ExecuteScalar().ToString();

                    SQLiteCommand cmd7 = new SQLiteCommand("SELECT linked6 FROM reviewrules WHERE keyword = '" + keywordBox.Text.ToString() + "' AND language = '" + languageBox.Text.ToString() + "' ", ObjConnection);
                    string linked6 = cmd7.ExecuteScalar().ToString();

                    SQLiteCommand cmd8 = new SQLiteCommand("SELECT linked7 FROM reviewrules WHERE keyword = '" + keywordBox.Text.ToString() + "' AND language = '" + languageBox.Text.ToString() + "' ", ObjConnection);
                    string linked7 = cmd8.ExecuteScalar().ToString();

                    SQLiteCommand cmd9 = new SQLiteCommand("SELECT linked8 FROM reviewrules WHERE keyword = '" + keywordBox.Text.ToString() + "' AND language = '" + languageBox.Text.ToString() + "' ", ObjConnection);
                    string linked8 = cmd9.ExecuteScalar().ToString();

                    SQLiteCommand cmd10 = new SQLiteCommand("SELECT linked9 FROM reviewrules WHERE keyword = '" + keywordBox.Text.ToString() + "' AND language = '" + languageBox.Text.ToString() + "' ", ObjConnection);
                    string linked9 = cmd10.ExecuteScalar().ToString();

                    SQLiteCommand cmd11 = new SQLiteCommand("SELECT linked10 FROM reviewrules WHERE keyword = '" + keywordBox.Text.ToString() + "' AND language = '" + languageBox.Text.ToString() + "' ", ObjConnection);
                    string linked10 = cmd11.ExecuteScalar().ToString();

                    string counttext = ("SELECT COUNT(*) FROM questions");
                    SQLiteCommand countcommand = new SQLiteCommand(counttext, ObjConnection);
                    string rowCount = countcommand.ExecuteScalar().ToString();
                    num = Convert.ToInt32(rowCount);

                    foreach (DataGridViewRow row in kryptonDataGridView1.Rows)
                    {
                        for (int i = 1; i <= num; i++)
                        {
                            if (row.Cells[1].Value.ToString() == linked1)
                            {
                                row.Cells[0].Value = true;
                            }
                        }

                        for (int i = 1; i <= num; i++)
                        {
                            if (row.Cells[1].Value.ToString() == linked2)
                            {
                                row.Cells[0].Value = true;
                            }
                        }

                        for (int i = 1; i <= num; i++)
                        {
                            if (row.Cells[1].Value.ToString() == linked3)
                            {
                                row.Cells[0].Value = true;
                            }
                        }

                        for (int i = 1; i <= num; i++)
                        {
                            if (row.Cells[1].Value.ToString() == linked4)
                            {
                                row.Cells[0].Value = true;
                            }
                        }

                        for (int i = 1; i <= num; i++)
                        {
                            if (row.Cells[1].Value.ToString() == linked5)
                            {
                                row.Cells[0].Value = true;
                            }
                        }

                        for (int i = 1; i <= num; i++)
                        {
                            if (row.Cells[1].Value.ToString() == linked6)
                            {
                                row.Cells[0].Value = true;
                            }
                        }

                        for (int i = 1; i <= num; i++)
                        {
                            if (row.Cells[1].Value.ToString() == linked7)
                            {
                                row.Cells[0].Value = true;
                            }
                        }

                        for (int i = 1; i <= num; i++)
                        {
                            if (row.Cells[1].Value.ToString() == linked8)
                            {
                                row.Cells[0].Value = true;
                            }
                        }

                        for (int i = 1; i <= num; i++)
                        {
                            if (row.Cells[1].Value.ToString() == linked9)
                            {
                                row.Cells[0].Value = true;
                            }
                        }

                        for (int i = 1; i <= num; i++)
                        {
                            if (row.Cells[1].Value.ToString() == linked10)
                            {
                                row.Cells[0].Value = true;
                            }
                        }
                    }

                    ObjConnection.Close();
                }

                catch
                {
                    DevExpress.XtraEditors.XtraMessageBox.Show("Unable to select item from the keyword list");
                    return;
                }
            }

            else
            {
                return;
            }
        }

        private void clearRulesButton_Click(object sender, EventArgs e)
        {
            languageCombo.Text = "";
            listView1.Clear();
            keywordBox.Text = "";
            languageBox.Text = "";
            ruleInfoBox.Text = "";
            foreach (DataGridViewRow row in kryptonDataGridView1.Rows)
            {
                for (int i = 1; i <= num; i++)
                {
                    row.Cells[0].Value = false;
                }
            }
        }

        private void deleteRulesButton_Click(object sender, EventArgs e)
        {
            if (keywordBox.Text != "")
            {
                try
                {
                    if (!Regex.IsMatch(keywordBox.Text, @"^[a-zA-Z0-9àáâãäåăćĉċčçèéêëēĕėěğĝġĥìíîïĩĭĵĺñńňòóôõöōŏőŕřśŝšşùúûüũūŭűŵŷýÿźżžÀÁÂÃÄÅĂßĆĈĊČÇÈÉÊËĒĔĖĚĞĜĠĤÌÍÎÏĨĬĹĴÑŃŇÒÓÔÕÖŌŎŐŔŘŚŜŠŞÙÚÛÜŨŪŬŰŴŶÝŸŹŻŽ\-_.,*?¡¿%{}()\[\]\\\/$:;+&@|~#""!&=<>^\b\s]*$"))
                        throw new Exception("Invalid characters in the keyword text box");
                    if (!Regex.IsMatch(languageBox.Text, @"^[a-zA-Z0-9àáâãäåăćĉċčçèéêëēĕėěğĝġĥìíîïĩĭĵĺñńňòóôõöōŏőŕřśŝšşùúûüũūŭűŵŷýÿźżžÀÁÂÃÄÅĂßĆĈĊČÇÈÉÊËĒĔĖĚĞĜĠĤÌÍÎÏĨĬĹĴÑŃŇÒÓÔÕÖŌŎŐŔŘŚŜŠŞÙÚÛÜŨŪŬŰŴŶÝŸŹŻŽ\-_.,*?¡¿%{}()\[\]\\\/$:;+&@|~#""!&=<>^\b\s]*$"))
                        throw new Exception("Invalid characters in the language text box");
                }
                catch (Exception ex)
                {
                    DevExpress.XtraEditors.XtraMessageBox.Show(ex.Message);
                    return;
                }

                int executed = 0;
                SQLiteConnection ObjConnection = new SQLiteConnection("Data Source=" + dPath + ";Version=3;New=False;Compress=True");

                string languageValue = languageBox.Text.ToString();

                try
                {
                    ObjConnection.Open();
                }

                catch
                {
                    DevExpress.XtraEditors.XtraMessageBox.Show("Could not connect to the database");
                    return;
                }

                try
                {
                    SQLiteCommand cmd = new SQLiteCommand("DELETE FROM reviewrules WHERE keyword = '" + keywordBox.Text.ToString() + "' AND language = '" + languageBox.Text.ToString() + "'", ObjConnection);
                    executed = cmd.ExecuteNonQuery();
                    ObjConnection.Close();
                }

                catch
                {
                    DevExpress.XtraEditors.XtraMessageBox.Show("Could not delete the keyword from the database");
                }

                if (executed == 1)
                {
                    DevExpress.XtraEditors.XtraMessageBox.Show("Keyword deleted from the review rules table");
                    
                    languageCombo.Text = "";
                    listView1.Clear();
                    keywordBox.Text = "";
                    languageBox.Text = "";
                    ruleInfoBox.Text = "";
                    foreach (DataGridViewRow row in kryptonDataGridView1.Rows)
                    {
                        for (int i = 1; i <= num; i++)
                        {
                            row.Cells[0].Value = false;
                        }
                    }

                    try
                    {
                        ObjConnection.Open();
                        SQLiteDataReader rdr = null;

                        languageCombo.Properties.Items.Clear();

                        SQLiteCommand cmd = new SQLiteCommand("SELECT DISTINCT language FROM reviewrules", ObjConnection);
                        rdr = cmd.ExecuteReader();

                        {
                            while (rdr.Read())
                            {
                                for (int i = 0; i < rdr.FieldCount; i++)
                                {
                                    languageCombo.Properties.Items.Add(rdr.GetValue(i).ToString());
                                }
                            }

                            languageCombo.Text = "";
                            languageCombo.Text = languageValue;
                        }
                    }

                    catch
                    {
                        DevExpress.XtraEditors.XtraMessageBox.Show("Unable to retrieve languages from the database");
                        return;
                    }

                    ObjConnection.Close();
                }
            }

            else
            {
                DevExpress.XtraEditors.XtraMessageBox.Show("Please select a keyword to delete from the database");
                return;
            }
        }

        private void saveRulesButton_Click(object sender, EventArgs e)
        {
            SQLiteConnection ObjConnection = new SQLiteConnection("Data Source=" + dPath + ";Version=3;New=False;Compress=True");

            int linked = 0;
            int unlinked = 0;

            foreach (DataGridViewRow row in kryptonDataGridView1.Rows)
            {
                if (row.Cells[0].Value == null || (bool)row.Cells[0].Value == false)
                {
                    unlinked++;
                }

                else
                {
                    linked++;
                }
            }

            if (linked == 0)
            {
                DevExpress.XtraEditors.XtraMessageBox.Show("You have selected no checklist items to link to this keyword");
                linked = 0;
                return;
            }

            if (linked >= 11)
            {
                DevExpress.XtraEditors.XtraMessageBox.Show("You have selected more than 10 checklist items to link to this keyword");
                linked = 0;
                return;
            }

            if (keywordBox.Text != "" && languageBox.Text != "" && ruleInfoBox.Text != "")
            {
                int keywordcount = 0;

                try
                {
                    string key = keywordBox.Text.ToString();
                    string lang = languageBox.Text.ToString();

                    ObjConnection.Open();
                    SQLiteCommand countcommand2 = new SQLiteCommand("SELECT COUNT(keyword) FROM reviewrules WHERE language = '" + lang + "' AND keyword = '"+ key +"' ", ObjConnection);
                    string rowCount2 = countcommand2.ExecuteScalar().ToString();
                    keywordcount = Convert.ToInt32(rowCount2);
                    ObjConnection.Close();
                }

                catch
                {
                    DevExpress.XtraEditors.XtraMessageBox.Show("Could not connect to the database");
                    return;
                }

                if (keywordcount == 1)
                {
                    try
                    {
                        if (!Regex.IsMatch(keywordBox.Text, @"^[a-zA-Z0-9àáâãäåăćĉċčçèéêëēĕėěğĝġĥìíîïĩĭĵĺñńňòóôõöōŏőŕřśŝšşùúûüũūŭűŵŷýÿźżžÀÁÂÃÄÅĂßĆĈĊČÇÈÉÊËĒĔĖĚĞĜĠĤÌÍÎÏĨĬĹĴÑŃŇÒÓÔÕÖŌŎŐŔŘŚŜŠŞÙÚÛÜŨŪŬŰŴŶÝŸŹŻŽ\-_.,*?¡¿%{}()\[\]\\\/$:;+&@|~#""!&=<>^\b\s]*$"))
                            throw new Exception("Invalid characters in the keyword text box");
                        if (!Regex.IsMatch(languageBox.Text, @"^[a-zA-Z0-9àáâãäåăćĉċčçèéêëēĕėěğĝġĥìíîïĩĭĵĺñńňòóôõöōŏőŕřśŝšşùúûüũūŭűŵŷýÿźżžÀÁÂÃÄÅĂßĆĈĊČÇÈÉÊËĒĔĖĚĞĜĠĤÌÍÎÏĨĬĹĴÑŃŇÒÓÔÕÖŌŎŐŔŘŚŜŠŞÙÚÛÜŨŪŬŰŴŶÝŸŹŻŽ\-_.,*?¡¿%{}()\[\]\\\/$:;+&@|~#""!&=<>^\b\s]*$"))
                            throw new Exception("Invalid characters in the language text box");
                        if (!Regex.IsMatch(ruleInfoBox.Text, @"^[a-zA-Z0-9àáâãäåăćĉċčçèéêëēĕėěğĝġĥìíîïĩĭĵĺñńňòóôõöōŏőŕřśŝšşùúûüũūŭűŵŷýÿźżžÀÁÂÃÄÅĂßĆĈĊČÇÈÉÊËĒĔĖĚĞĜĠĤÌÍÎÏĨĬĹĴÑŃŇÒÓÔÕÖŌŎŐŔŘŚŜŠŞÙÚÛÜŨŪŬŰŴŶÝŸŹŻŽ\-_'.,*?¡¿%{}()\[\]\\\/$:;+&@|~#""!&=<>^\b\s]*$"))
                            throw new Exception("Invalid characters in the rule info text box");
                    }
                    catch (Exception ex)
                    {
                        DevExpress.XtraEditors.XtraMessageBox.Show(ex.Message);
                        return;
                    }

                    string linked1Text = "";
                    string linked2Text = "";
                    string linked3Text = "";
                    string linked4Text = "";
                    string linked5Text = "";
                    string linked6Text = "";
                    string linked7Text = "";
                    string linked8Text = "";
                    string linked9Text = "";
                    string linked10Text = "";

                    try
                    {
                        string key = keywordBox.Text.ToString();
                        string lang = languageBox.Text.ToString();

                        foreach (DataGridViewRow row in kryptonDataGridView1.Rows)
                        {
                            if (row.Cells[0].Value != null && (bool)row.Cells[0].Value == true)
                            {
                                linked1Text = row.Cells[1].Value.ToString();
                            }
                        }

                        foreach (DataGridViewRow row in kryptonDataGridView1.Rows)
                        {
                            if (row.Cells[1].Value.ToString() != linked1Text && row.Cells[0].Value != null && (bool)row.Cells[0].Value == true)
                            {
                                linked2Text = row.Cells[1].Value.ToString();
                            }
                        }

                        foreach (DataGridViewRow row in kryptonDataGridView1.Rows)
                        {
                            if (row.Cells[1].Value.ToString() != linked1Text && row.Cells[1].Value.ToString() != linked2Text && row.Cells[0].Value != null && (bool)row.Cells[0].Value == true)
                            {
                                linked3Text = row.Cells[1].Value.ToString();
                            }
                        }

                        foreach (DataGridViewRow row in kryptonDataGridView1.Rows)
                        {
                            if (row.Cells[1].Value.ToString() != linked1Text && row.Cells[1].Value.ToString() != linked2Text && row.Cells[1].Value.ToString() != linked3Text && row.Cells[0].Value != null && (bool)row.Cells[0].Value == true)
                            {
                                linked4Text = row.Cells[1].Value.ToString();
                            }
                        }

                        foreach (DataGridViewRow row in kryptonDataGridView1.Rows)
                        {
                            if (row.Cells[1].Value.ToString() != linked1Text && row.Cells[1].Value.ToString() != linked2Text && row.Cells[1].Value.ToString() != linked3Text && row.Cells[1].Value.ToString() != linked4Text && row.Cells[0].Value != null && (bool)row.Cells[0].Value == true)
                            {
                                linked5Text = row.Cells[1].Value.ToString();
                            }
                        }

                        foreach (DataGridViewRow row in kryptonDataGridView1.Rows)
                        {
                            if (row.Cells[1].Value.ToString() != linked1Text && row.Cells[1].Value.ToString() != linked2Text && row.Cells[1].Value.ToString() != linked3Text && row.Cells[1].Value.ToString() != linked4Text && row.Cells[1].Value.ToString() != linked5Text && row.Cells[0].Value != null && (bool)row.Cells[0].Value == true)
                            {
                                linked6Text = row.Cells[1].Value.ToString();
                            }
                        }

                        foreach (DataGridViewRow row in kryptonDataGridView1.Rows)
                        {
                            if (row.Cells[1].Value.ToString() != linked1Text && row.Cells[1].Value.ToString() != linked2Text && row.Cells[1].Value.ToString() != linked3Text && row.Cells[1].Value.ToString() != linked4Text && row.Cells[1].Value.ToString() != linked5Text && row.Cells[1].Value.ToString() != linked6Text && row.Cells[0].Value != null && (bool)row.Cells[0].Value == true)
                            {
                                linked7Text = row.Cells[1].Value.ToString();
                            }
                        }

                        foreach (DataGridViewRow row in kryptonDataGridView1.Rows)
                        {
                            if (row.Cells[1].Value.ToString() != linked1Text && row.Cells[1].Value.ToString() != linked2Text && row.Cells[1].Value.ToString() != linked3Text && row.Cells[1].Value.ToString() != linked4Text && row.Cells[1].Value.ToString() != linked5Text && row.Cells[1].Value.ToString() != linked6Text && row.Cells[1].Value.ToString() != linked7Text && row.Cells[0].Value != null && (bool)row.Cells[0].Value == true)
                            {
                                linked8Text = row.Cells[1].Value.ToString();
                            }
                        }

                        foreach (DataGridViewRow row in kryptonDataGridView1.Rows)
                        {
                            if (row.Cells[1].Value.ToString() != linked1Text && row.Cells[1].Value.ToString() != linked2Text && row.Cells[1].Value.ToString() != linked3Text && row.Cells[1].Value.ToString() != linked4Text && row.Cells[1].Value.ToString() != linked5Text && row.Cells[1].Value.ToString() != linked6Text && row.Cells[1].Value.ToString() != linked7Text && row.Cells[1].Value.ToString() != linked8Text && row.Cells[0].Value != null && (bool)row.Cells[0].Value == true)
                            {
                                linked9Text = row.Cells[1].Value.ToString();
                            }
                        }

                        foreach (DataGridViewRow row in kryptonDataGridView1.Rows)
                        {
                            if (row.Cells[1].Value.ToString() != linked1Text && row.Cells[1].Value.ToString() != linked2Text && row.Cells[1].Value.ToString() != linked3Text && row.Cells[1].Value.ToString() != linked4Text && row.Cells[1].Value.ToString() != linked5Text && row.Cells[1].Value.ToString() != linked6Text && row.Cells[1].Value.ToString() != linked7Text && row.Cells[1].Value.ToString() != linked8Text && row.Cells[1].Value.ToString() != linked9Text && row.Cells[0].Value != null && (bool)row.Cells[0].Value == true)
                            {
                                linked10Text = row.Cells[1].Value.ToString();
                            }
                        }
                    }

                    catch
                    {
                        DevExpress.XtraEditors.XtraMessageBox.Show("Could not connect to the database");
                        return;
                    }

                    SQLiteConnection sqlite_conn;

                    SQLiteCommand sqlite_cmd;
                    SQLiteParameter keyword;
                    SQLiteParameter language;
                    SQLiteParameter info;
                    SQLiteParameter linked1;
                    SQLiteParameter linked2;
                    SQLiteParameter linked3;
                    SQLiteParameter linked4;
                    SQLiteParameter linked5;
                    SQLiteParameter linked6;
                    SQLiteParameter linked7;
                    SQLiteParameter linked8;
                    SQLiteParameter linked9;
                    SQLiteParameter linked10;

                    sqlite_conn = new SQLiteConnection("Data Source=" + dPath + ";Version=3;New=False;Compress=True;FailIfMissing=True");

                    sqlite_cmd = sqlite_conn.CreateCommand();
                    sqlite_cmd.CommandText = ("UPDATE reviewrules SET keyword = @keyword, language = @language, info = @info, linked1 = @linked1, linked2 = @linked2, linked3 = @linked3, linked4 = @linked4, linked5 = @linked5, linked6 = @linked6, linked7 = @linked7, linked8 = @linked8, linked9 = @linked9, linked10 = @linked10 WHERE keyword = '" + keywordBox.Text.ToString() + "' AND language = '" + languageBox.Text.ToString() + "' ");

                    keyword = new SQLiteParameter();
                    keyword.ParameterName = "@keyword";
                    keyword.DbType = DbType.String;
                    keyword.Size = 100;
                    keyword.Direction = ParameterDirection.Input;
                    keyword.Value = keywordBox.Text.ToString();

                    language = new SQLiteParameter();
                    language.ParameterName = "@language";
                    language.DbType = DbType.String;
                    language.Size = 100;
                    language.Direction = ParameterDirection.Input;
                    language.Value = languageBox.Text.ToString();

                    info = new SQLiteParameter();
                    info.ParameterName = "@info";
                    info.DbType = DbType.String;
                    info.Size = 5000;
                    info.Direction = ParameterDirection.Input;
                    info.Value = ruleInfoBox.Text.ToString();

                    linked1 = new SQLiteParameter();
                    linked1.ParameterName = "@linked1";
                    linked1.DbType = DbType.String;
                    linked1.Size = 5000;
                    linked1.Direction = ParameterDirection.Input;
                    linked1.Value = linked1Text;

                    linked2 = new SQLiteParameter();
                    linked2.ParameterName = "@linked2";
                    linked2.DbType = DbType.String;
                    linked2.Size = 5000;
                    linked2.Direction = ParameterDirection.Input;
                    linked2.Value = linked2Text;

                    linked3 = new SQLiteParameter();
                    linked3.ParameterName = "@linked3";
                    linked3.DbType = DbType.String;
                    linked3.Size = 5000;
                    linked3.Direction = ParameterDirection.Input;
                    linked3.Value = linked3Text;

                    linked4 = new SQLiteParameter();
                    linked4.ParameterName = "@linked4";
                    linked4.DbType = DbType.String;
                    linked4.Size = 5000;
                    linked4.Direction = ParameterDirection.Input;
                    linked4.Value = linked4Text;

                    linked5 = new SQLiteParameter();
                    linked5.ParameterName = "@linked5";
                    linked5.DbType = DbType.String;
                    linked5.Size = 5000;
                    linked5.Direction = ParameterDirection.Input;
                    linked5.Value = linked5Text;

                    linked6 = new SQLiteParameter();
                    linked6.ParameterName = "@linked6";
                    linked6.DbType = DbType.String;
                    linked6.Size = 5000;
                    linked6.Direction = ParameterDirection.Input;
                    linked6.Value = linked6Text;

                    linked7 = new SQLiteParameter();
                    linked7.ParameterName = "@linked7";
                    linked7.DbType = DbType.String;
                    linked7.Size = 5000;
                    linked7.Direction = ParameterDirection.Input;
                    linked7.Value = linked7Text;

                    linked8 = new SQLiteParameter();
                    linked8.ParameterName = "@linked8";
                    linked8.DbType = DbType.String;
                    linked8.Size = 5000;
                    linked8.Direction = ParameterDirection.Input;
                    linked8.Value = linked8Text;

                    linked9 = new SQLiteParameter();
                    linked9.ParameterName = "@linked9";
                    linked9.DbType = DbType.String;
                    linked9.Size = 5000;
                    linked9.Direction = ParameterDirection.Input;
                    linked9.Value = linked9Text;

                    linked10 = new SQLiteParameter();
                    linked10.ParameterName = "@linked10";
                    linked10.DbType = DbType.String;
                    linked10.Size = 5000;
                    linked10.Direction = ParameterDirection.Input;
                    linked10.Value = linked10Text;

                    sqlite_cmd.Parameters.Add(keyword);
                    sqlite_cmd.Parameters.Add(language);
                    sqlite_cmd.Parameters.Add(info);
                    sqlite_cmd.Parameters.Add(linked1);
                    sqlite_cmd.Parameters.Add(linked2);
                    sqlite_cmd.Parameters.Add(linked3);
                    sqlite_cmd.Parameters.Add(linked4);
                    sqlite_cmd.Parameters.Add(linked5);
                    sqlite_cmd.Parameters.Add(linked6);
                    sqlite_cmd.Parameters.Add(linked7);
                    sqlite_cmd.Parameters.Add(linked8);
                    sqlite_cmd.Parameters.Add(linked9);
                    sqlite_cmd.Parameters.Add(linked10);

                    try
                    {
                        sqlite_conn.Open();
                    }

                    catch
                    {
                        DevExpress.XtraEditors.XtraMessageBox.Show("Could not connect to the database");
                        return;
                    }

                    int executed = 0;

                    try
                    {
                        executed = sqlite_cmd.ExecuteNonQuery();
                    }

                    catch
                    {
                        DevExpress.XtraEditors.XtraMessageBox.Show("Unable to insert data into review rules table");
                        return;
                    }

                    sqlite_conn.Close();

                    if (executed == 1)
                    {
                        SQLiteConnection ObjConnection2 = new SQLiteConnection("Data Source=" + dPath + ";Version=3;New=False;Compress=True");

                        ObjConnection2.Open();
                        SQLiteDataReader rdr = null;

                        languageCombo.Properties.Items.Clear();

                        SQLiteCommand cmd = new SQLiteCommand("SELECT DISTINCT language FROM reviewrules", ObjConnection2);
                        rdr = cmd.ExecuteReader();

                        {
                            while (rdr.Read())
                            {
                                for (int i = 0; i < rdr.FieldCount; i++)
                                {
                                    languageCombo.Properties.Items.Add(rdr.GetValue(i).ToString());
                                }
                            }
                        }
                        ObjConnection2.Close();
                        DevExpress.XtraEditors.XtraMessageBox.Show("Review rules table updated successfully");
                    }
                }

                else
                {
                    try
                    {
                        if (!Regex.IsMatch(keywordBox.Text, @"^[a-zA-Z0-9àáâãäåăćĉċčçèéêëēĕėěğĝġĥìíîïĩĭĵĺñńňòóôõöōŏőŕřśŝšşùúûüũūŭűŵŷýÿźżžÀÁÂÃÄÅĂßĆĈĊČÇÈÉÊËĒĔĖĚĞĜĠĤÌÍÎÏĨĬĹĴÑŃŇÒÓÔÕÖŌŎŐŔŘŚŜŠŞÙÚÛÜŨŪŬŰŴŶÝŸŹŻŽ\-_.,*?¡¿%{}()\[\]\\\/$:;+&@|~#""!&=<>^\b\s]*$"))
                            throw new Exception("Invalid characters in the keyword text box");
                        if (!Regex.IsMatch(languageBox.Text, @"^[a-zA-Z0-9àáâãäåăćĉċčçèéêëēĕėěğĝġĥìíîïĩĭĵĺñńňòóôõöōŏőŕřśŝšşùúûüũūŭűŵŷýÿźżžÀÁÂÃÄÅĂßĆĈĊČÇÈÉÊËĒĔĖĚĞĜĠĤÌÍÎÏĨĬĹĴÑŃŇÒÓÔÕÖŌŎŐŔŘŚŜŠŞÙÚÛÜŨŪŬŰŴŶÝŸŹŻŽ\-_.,*?¡¿%{}()\[\]\\\/$:;+&@|~#""!&=<>^\b\s]*$"))
                            throw new Exception("Invalid characters in the language text box");
                        if (!Regex.IsMatch(ruleInfoBox.Text, @"^[a-zA-Z0-9àáâãäåăćĉċčçèéêëēĕėěğĝġĥìíîïĩĭĵĺñńňòóôõöōŏőŕřśŝšşùúûüũūŭűŵŷýÿźżžÀÁÂÃÄÅĂßĆĈĊČÇÈÉÊËĒĔĖĚĞĜĠĤÌÍÎÏĨĬĹĴÑŃŇÒÓÔÕÖŌŎŐŔŘŚŜŠŞÙÚÛÜŨŪŬŰŴŶÝŸŹŻŽ\-_'.,*?¡¿%{}()\[\]\\\/$:;+&@|~#""!&=<>^\b\s]*$"))
                            throw new Exception("Invalid characters in the rule info text box");
                    }
                    catch (Exception ex)
                    {
                        DevExpress.XtraEditors.XtraMessageBox.Show(ex.Message);
                        return;
                    }

                    string linked1Text = "";
                    string linked2Text = "";
                    string linked3Text = "";
                    string linked4Text = "";
                    string linked5Text = "";
                    string linked6Text = "";
                    string linked7Text = "";
                    string linked8Text = "";
                    string linked9Text = "";
                    string linked10Text = "";

                    try
                    {
                        string key = keywordBox.Text.ToString();
                        string lang = languageBox.Text.ToString();

                        foreach (DataGridViewRow row in kryptonDataGridView1.Rows)
                        {
                            if (row.Cells[0].Value != null && (bool)row.Cells[0].Value == true)
                            {
                                linked1Text = row.Cells[1].Value.ToString();
                            }
                        }

                        foreach (DataGridViewRow row in kryptonDataGridView1.Rows)
                        {
                            if (row.Cells[1].Value.ToString() != linked1Text && row.Cells[0].Value != null && (bool)row.Cells[0].Value == true)
                            {
                                linked2Text = row.Cells[1].Value.ToString();
                            }
                        }

                        foreach (DataGridViewRow row in kryptonDataGridView1.Rows)
                        {
                            if (row.Cells[1].Value.ToString() != linked1Text && row.Cells[1].Value.ToString() != linked2Text && row.Cells[0].Value != null && (bool)row.Cells[0].Value == true)
                            {
                                linked3Text = row.Cells[1].Value.ToString();
                            }
                        }

                        foreach (DataGridViewRow row in kryptonDataGridView1.Rows)
                        {
                            if (row.Cells[1].Value.ToString() != linked1Text && row.Cells[1].Value.ToString() != linked2Text && row.Cells[1].Value.ToString() != linked3Text && row.Cells[0].Value != null && (bool)row.Cells[0].Value == true)
                            {
                                linked4Text = row.Cells[1].Value.ToString();
                            }
                        }

                        foreach (DataGridViewRow row in kryptonDataGridView1.Rows)
                        {
                            if (row.Cells[1].Value.ToString() != linked1Text && row.Cells[1].Value.ToString() != linked2Text && row.Cells[1].Value.ToString() != linked3Text && row.Cells[1].Value.ToString() != linked4Text && row.Cells[0].Value != null && (bool)row.Cells[0].Value == true)
                            {
                                linked5Text = row.Cells[1].Value.ToString();
                            }
                        }

                        foreach (DataGridViewRow row in kryptonDataGridView1.Rows)
                        {
                            if (row.Cells[1].Value.ToString() != linked1Text && row.Cells[1].Value.ToString() != linked2Text && row.Cells[1].Value.ToString() != linked3Text && row.Cells[1].Value.ToString() != linked4Text && row.Cells[1].Value.ToString() != linked5Text && row.Cells[0].Value != null && (bool)row.Cells[0].Value == true)
                            {
                                linked6Text = row.Cells[1].Value.ToString();
                            }
                        }

                        foreach (DataGridViewRow row in kryptonDataGridView1.Rows)
                        {
                            if (row.Cells[1].Value.ToString() != linked1Text && row.Cells[1].Value.ToString() != linked2Text && row.Cells[1].Value.ToString() != linked3Text && row.Cells[1].Value.ToString() != linked4Text && row.Cells[1].Value.ToString() != linked5Text && row.Cells[1].Value.ToString() != linked6Text && row.Cells[0].Value != null && (bool)row.Cells[0].Value == true)
                            {
                                linked7Text = row.Cells[1].Value.ToString();
                            }
                        }

                        foreach (DataGridViewRow row in kryptonDataGridView1.Rows)
                        {
                            if (row.Cells[1].Value.ToString() != linked1Text && row.Cells[1].Value.ToString() != linked2Text && row.Cells[1].Value.ToString() != linked3Text && row.Cells[1].Value.ToString() != linked4Text && row.Cells[1].Value.ToString() != linked5Text && row.Cells[1].Value.ToString() != linked6Text && row.Cells[1].Value.ToString() != linked7Text && row.Cells[0].Value != null && (bool)row.Cells[0].Value == true)
                            {
                                linked8Text = row.Cells[1].Value.ToString();
                            }
                        }

                        foreach (DataGridViewRow row in kryptonDataGridView1.Rows)
                        {
                            if (row.Cells[1].Value.ToString() != linked1Text && row.Cells[1].Value.ToString() != linked2Text && row.Cells[1].Value.ToString() != linked3Text && row.Cells[1].Value.ToString() != linked4Text && row.Cells[1].Value.ToString() != linked5Text && row.Cells[1].Value.ToString() != linked6Text && row.Cells[1].Value.ToString() != linked7Text && row.Cells[1].Value.ToString() != linked8Text && row.Cells[0].Value != null && (bool)row.Cells[0].Value == true)
                            {
                                linked9Text = row.Cells[1].Value.ToString();
                            }
                        }

                        foreach (DataGridViewRow row in kryptonDataGridView1.Rows)
                        {
                            if (row.Cells[1].Value.ToString() != linked1Text && row.Cells[1].Value.ToString() != linked2Text && row.Cells[1].Value.ToString() != linked3Text && row.Cells[1].Value.ToString() != linked4Text && row.Cells[1].Value.ToString() != linked5Text && row.Cells[1].Value.ToString() != linked6Text && row.Cells[1].Value.ToString() != linked7Text && row.Cells[1].Value.ToString() != linked8Text && row.Cells[1].Value.ToString() != linked9Text && row.Cells[0].Value != null && (bool)row.Cells[0].Value == true)
                            {
                                linked10Text = row.Cells[1].Value.ToString();
                            }
                        }
                    }

                    catch
                    {
                        DevExpress.XtraEditors.XtraMessageBox.Show("Could not connect to the database");
                        return;
                    }

                    SQLiteConnection sqlite_conn;

                    SQLiteCommand sqlite_cmd;
                    SQLiteParameter keyword;
                    SQLiteParameter language;
                    SQLiteParameter info;
                    SQLiteParameter linked1;
                    SQLiteParameter linked2;
                    SQLiteParameter linked3;
                    SQLiteParameter linked4;
                    SQLiteParameter linked5;
                    SQLiteParameter linked6;
                    SQLiteParameter linked7;
                    SQLiteParameter linked8;
                    SQLiteParameter linked9;
                    SQLiteParameter linked10;

                    sqlite_conn = new SQLiteConnection("Data Source=" + dPath + ";Version=3;New=False;Compress=True;FailIfMissing=True");

                    sqlite_cmd = sqlite_conn.CreateCommand();
                    sqlite_cmd.CommandText = ("INSERT into reviewrules (keyword, language, info, linked1, linked2, linked3, linked4, linked5, linked6, linked7, linked8, linked9, linked10) VALUES (@keyword, @language, @info, @linked1, @linked2, @linked3, @linked4, @linked5, @linked6, @linked7, @linked8, @linked9, @linked10)");

                    keyword = new SQLiteParameter();
                    keyword.ParameterName = "@keyword";
                    keyword.DbType = DbType.String;
                    keyword.Size = 100;
                    keyword.Direction = ParameterDirection.Input;
                    keyword.Value = keywordBox.Text.ToString();

                    language = new SQLiteParameter();
                    language.ParameterName = "@language";
                    language.DbType = DbType.String;
                    language.Size = 100;
                    language.Direction = ParameterDirection.Input;
                    language.Value = languageBox.Text.ToString();

                    info = new SQLiteParameter();
                    info.ParameterName = "@info";
                    info.DbType = DbType.String;
                    info.Size = 5000;
                    info.Direction = ParameterDirection.Input;
                    info.Value = ruleInfoBox.Text.ToString();

                    linked1 = new SQLiteParameter();
                    linked1.ParameterName = "@linked1";
                    linked1.DbType = DbType.String;
                    linked1.Size = 5000;
                    linked1.Direction = ParameterDirection.Input;
                    linked1.Value = linked1Text;

                    linked2 = new SQLiteParameter();
                    linked2.ParameterName = "@linked2";
                    linked2.DbType = DbType.String;
                    linked2.Size = 5000;
                    linked2.Direction = ParameterDirection.Input;
                    linked2.Value = linked2Text;

                    linked3 = new SQLiteParameter();
                    linked3.ParameterName = "@linked3";
                    linked3.DbType = DbType.String;
                    linked3.Size = 5000;
                    linked3.Direction = ParameterDirection.Input;
                    linked3.Value = linked3Text;

                    linked4 = new SQLiteParameter();
                    linked4.ParameterName = "@linked4";
                    linked4.DbType = DbType.String;
                    linked4.Size = 5000;
                    linked4.Direction = ParameterDirection.Input;
                    linked4.Value = linked4Text;

                    linked5 = new SQLiteParameter();
                    linked5.ParameterName = "@linked5";
                    linked5.DbType = DbType.String;
                    linked5.Size = 5000;
                    linked5.Direction = ParameterDirection.Input;
                    linked5.Value = linked5Text;

                    linked6 = new SQLiteParameter();
                    linked6.ParameterName = "@linked6";
                    linked6.DbType = DbType.String;
                    linked6.Size = 5000;
                    linked6.Direction = ParameterDirection.Input;
                    linked6.Value = linked6Text;

                    linked7 = new SQLiteParameter();
                    linked7.ParameterName = "@linked7";
                    linked7.DbType = DbType.String;
                    linked7.Size = 5000;
                    linked7.Direction = ParameterDirection.Input;
                    linked7.Value = linked7Text;

                    linked8 = new SQLiteParameter();
                    linked8.ParameterName = "@linked8";
                    linked8.DbType = DbType.String;
                    linked8.Size = 5000;
                    linked8.Direction = ParameterDirection.Input;
                    linked8.Value = linked8Text;

                    linked9 = new SQLiteParameter();
                    linked9.ParameterName = "@linked9";
                    linked9.DbType = DbType.String;
                    linked9.Size = 5000;
                    linked9.Direction = ParameterDirection.Input;
                    linked9.Value = linked9Text;

                    linked10 = new SQLiteParameter();
                    linked10.ParameterName = "@linked10";
                    linked10.DbType = DbType.String;
                    linked10.Size = 5000;
                    linked10.Direction = ParameterDirection.Input;
                    linked10.Value = linked10Text;

                    sqlite_cmd.Parameters.Add(keyword);
                    sqlite_cmd.Parameters.Add(language);
                    sqlite_cmd.Parameters.Add(info);
                    sqlite_cmd.Parameters.Add(linked1);
                    sqlite_cmd.Parameters.Add(linked2);
                    sqlite_cmd.Parameters.Add(linked3);
                    sqlite_cmd.Parameters.Add(linked4);
                    sqlite_cmd.Parameters.Add(linked5);
                    sqlite_cmd.Parameters.Add(linked6);
                    sqlite_cmd.Parameters.Add(linked7);
                    sqlite_cmd.Parameters.Add(linked8);
                    sqlite_cmd.Parameters.Add(linked9);
                    sqlite_cmd.Parameters.Add(linked10);

                    string languageValue = languageBox.Text.ToString();

                    try
                    {
                        sqlite_conn.Open();
                    }

                    catch
                    {
                        DevExpress.XtraEditors.XtraMessageBox.Show("Could not connect to the database");
                        return;
                    }

                    int executed = 0;

                    try
                    {
                        executed = sqlite_cmd.ExecuteNonQuery();
                    }

                    catch
                    {
                        DevExpress.XtraEditors.XtraMessageBox.Show("Unable to insert data into review rules table");
                        return;
                    }

                    sqlite_conn.Close();

                    if (executed == 1)
                    {
                        SQLiteConnection ObjConnection3 = new SQLiteConnection("Data Source=" + dPath + ";Version=3;New=False;Compress=True");

                        ObjConnection3.Open();
                        SQLiteDataReader rdr = null;

                        languageCombo.Properties.Items.Clear();

                        SQLiteCommand cmd = new SQLiteCommand("SELECT DISTINCT language FROM reviewrules", ObjConnection3);
                        rdr = cmd.ExecuteReader();

                        {
                            while (rdr.Read())
                            {
                                for (int i = 0; i < rdr.FieldCount; i++)
                                {
                                    languageCombo.Properties.Items.Add(rdr.GetValue(i).ToString());
                                }
                            }

                            languageCombo.Text = "";
                            languageCombo.Text = languageValue;
                        }
                        ObjConnection3.Close();
                        DevExpress.XtraEditors.XtraMessageBox.Show("Review rules table updated successfully");
                    }
                }
            }

            else
            {
                DevExpress.XtraEditors.XtraMessageBox.Show("Please complete the keyword and language fields");
                return;
            }
        }
    }
}
