﻿/*Agnitio security code review tool.
Copyright (C) 2011 David Rook

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Data.SQLite;
using System.Text.RegularExpressions;

namespace Agnitio
{
    public partial class configuration : Form
    {
        public configuration()
        {
            InitializeComponent();
        }

        Form1 mainCode = new Form1();
        string progPathText = Form1.progPath.ToString();

        public void configuration_Load(object sender, EventArgs e)
        {
            if (Form1.dPath.ToString() != "" && Form1.rPath.ToString() != "")
            {
                dPathText.Text = Form1.dPath.ToString();
                rPathText.Text = Form1.rPath.ToString();
            }

            else
            {
                DevExpress.XtraEditors.XtraMessageBox.Show("Database and report path must not be blank");
                return;
            }
        }

        private void dPathText_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (!Regex.IsMatch(e.KeyChar.ToString(), @"^ . $"))
                e.Handled = true;
        }

        private void rPathText_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (!Regex.IsMatch(e.KeyChar.ToString(), @"^ . $"))
                e.Handled = true;
        }

        private void loadDBPath()
        {
            try
            {
                //This opens the "browse for report" dialog box, restricts it to html only, goes straight to reports directory

                OpenFileDialog fDialog = new OpenFileDialog();

                fDialog.Title = "Please select a database file";

                fDialog.Filter = "SQLite Files|*.s3db";

                fDialog.InitialDirectory = @"" + progPathText + "\\Agnitio";

                fDialog.AddExtension = true;

                fDialog.CheckFileExists = true;

                fDialog.CheckPathExists = true;

                if (fDialog.ShowDialog() == DialogResult.OK)
                {

                    if (!Regex.IsMatch(fDialog.FileName.ToString(), @"^(([a-zA-ZàáâãäåăćĉċčçèéêëēĕėěğĝġĥìíîïĩĭĵĺñńňòóôõöōŏőŕřśŝšşùúûüũūŭűŵŷýÿźżžÀÁÂÃÄÅĂßĆĈĊČÇÈÉÊËĒĔĖĚĞĜĠĤÌÍÎÏĨĬĹĴÑŃŇÒÓÔÕÖŌŎŐŔŘŚŜŠŞÙÚÛÜŨŪŬŰŴŶÝŸŹŻŽ]\:)|(\\))(\\{1}|((\\{1})[^\\]([^/:*?<>""|]*))+)$"))
                    {
                        DevExpress.XtraEditors.XtraMessageBox.Show("Invalid characters in database path field");
                        return;
                    }

                    else
                    {
                        dPathText.Text = fDialog.FileName.ToString();
                    }
                }

            }

            catch
            {
                DevExpress.XtraEditors.XtraMessageBox.Show("Unable to open file dialog box");
                return;
            }
        }

        private void loadRPPath()
        {
            if (folderBrowserDialog1.ShowDialog() == DialogResult.OK)
            {
                if (!Regex.IsMatch(folderBrowserDialog1.SelectedPath.ToString(), @"^(([a-zA-ZàáâãäåăćĉċčçèéêëēĕėěğĝġĥìíîïĩĭĵĺñńňòóôõöōŏőŕřśŝšşùúûüũūŭűŵŷýÿźżžÀÁÂÃÄÅĂßĆĈĊČÇÈÉÊËĒĔĖĚĞĜĠĤÌÍÎÏĨĬĹĴÑŃŇÒÓÔÕÖŌŎŐŔŘŚŜŠŞÙÚÛÜŨŪŬŰŴŶÝŸŹŻŽ]\:)|(\\))(\\{1}|((\\{1})[^\\]([^/:*?<>""|]*))+)$"))
                {
                    DevExpress.XtraEditors.XtraMessageBox.Show("Invalid characters in report path field");
                    return;
                }

                else
                {
                    rPathText.Text = folderBrowserDialog1.SelectedPath;
                }
            }
        }

        private void savePaths()
        {
            if (dPathText.Text != "" && rPathText.Text != "")
            {

                SQLiteConnection sqlite_conn;

                SQLiteCommand sqlite_cmd;

                sqlite_conn = new SQLiteConnection("Data Source=" + progPathText + "\\Agnitio\\configuration.s3db;Version=3;New=False;Compress=True;FailIfMissing=True");

                sqlite_cmd = sqlite_conn.CreateCommand();

                sqlite_cmd.CommandText = ("UPDATE path SET databasepath = '" + dPathText.Text + "', reportpath = '" + rPathText.Text + "' ");

                try
                {
                    sqlite_conn.Open();
                }

                catch
                {
                    DevExpress.XtraEditors.XtraMessageBox.Show("Could not connect to the checklist database");
                    return;
                }

                int executed = 0;

                try
                {
                    executed = sqlite_cmd.ExecuteNonQuery();
                }

                catch
                {
                    DevExpress.XtraEditors.XtraMessageBox.Show("Unable to update configuration data");
                    return;
                }

                sqlite_conn.Close();

                if (executed == 1)
                {
                    DevExpress.XtraEditors.XtraMessageBox.Show("Configuration data updated, if you updated the database path please restart Agnitio");
                    Form1.dPath = dPathText.Text;
                    Form1.rPath = rPathText.Text;

                    dPathText.Text = "";
                    rPathText.Text = "";
                }
            }

            else
            {
                DevExpress.XtraEditors.XtraMessageBox.Show("Database and report path must not be blank");
                return;
            }
        }

        private void dbConfBrowse_Click(object sender, EventArgs e)
        {
            loadDBPath();
        }

        private void rpConfBrowse_Click(object sender, EventArgs e)
        {
            loadRPPath();
        }

        private void cSaveButton_Click(object sender, EventArgs e)
        {
            savePaths();
        }

        private void cCloseButton_Click(object sender, EventArgs e)
        {
            this.Close();
        }
    }
}
